// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <ctype.h>

#include <anna/core/util/CommandLine.hpp>
#include <anna/core/tracing/Logger.hpp>

#include <anna/comm/Network.hpp>
#include <anna/comm/IndexedDelivery.hpp>
#include <anna/comm/ServerSocket.hpp>
#include <anna/comm/Device.hpp>

#include <anna/http/Transport.hpp>

#include "Application.hpp"
#include "Context.hpp"

using namespace std;
using namespace anna;
using namespace test;

http4comm::Application::Application () : 
   comm::Application ("http_rserver", "Arithmetic HTTP server", "1.0") ,
   a_timeController ((Millisecond)30000, (Millisecond)1000),
   a_communicator (NULL)
{
   CommandLine& commandLine (CommandLine::instantiate ());
      
   commandLine.add ("p", CommandLine::Argument::Mandatory, "Listen port for requests");
   commandLine.add ("a", CommandLine::Argument::Mandatory, "Listen IP address for requests");
   commandLine.add ("mode", CommandLine::Argument::Mandatory, "Delivery mode, (S)trict or (F)lexible");
   commandLine.add ("as", CommandLine::Argument::Mandatory, "Arithmetic servers IP");
   commandLine.add ("pp", CommandLine::Argument::Mandatory, "Port for + operation");
   commandLine.add ("pm", CommandLine::Argument::Mandatory, "Port for - operation");
   commandLine.add ("px", CommandLine::Argument::Mandatory, "Port for * operation");
   commandLine.add ("pd", CommandLine::Argument::Mandatory, "Port for / operation");
   commandLine.add ("trace", CommandLine::Argument::Optional, "Trace level (debug, warning, error,...)");
   commandLine.add ("clone", CommandLine::Argument::Optional, "Clone mode", false);
}

void http4comm::Application::initialize () 
   throw (RuntimeException)
{
   using namespace anna::comm;

   CommandLine& cl (CommandLine::instantiate ());

   Communicator::WorkMode::_v workMode (Communicator::WorkMode::Single);

   if (cl.exists ("trace"))
      Logger::setLevel (Logger::asLevel (cl.getValue ("trace")));

   if (cl.exists ("clone"))
      workMode = Communicator::WorkMode::Clone;

   a_communicator = new Communicator (workMode);

   int port = cl.getIntegerValue ("p");
   Network& network = Network::instantiate ();

   const char* ip = cl.getValue ("a");
   const char mode = toupper (*cl.getValue ("mode"));

   const Device* device = Network::instantiate ().find (Device::asAddress (ip));

   a_serverSocket = new ServerSocket (INetAddress (device, port), false, &http::Transport::getFactory ());
   a_serverSocket->setReceiverFactory (a_acceptorFactory);

   a_service = new comm::IndexedDelivery ("Service_Arithmetic", true, (mode == 'S') ? comm::IndexedDelivery::Mode::Strict: comm::IndexedDelivery::Mode::Flexible);

   ip = cl.getValue ("as");

   port = cl.getIntegerValue ("pp");
   a_service->attach (network.createServer (ip, port, true, a_reactorFactory));

   port = cl.getIntegerValue ("pm");
   a_service->attach (network.createServer (ip, port, true, a_reactorFactory));

   port = cl.getIntegerValue ("px");
   a_service->attach (network.createServer (ip, port, true, a_reactorFactory));

   port = cl.getIntegerValue ("pd");
   a_service->attach (network.createServer (ip, port, true, a_reactorFactory));

   a_communicator->attach (a_service);
   a_communicator->attach (a_serverSocket);
   
   a_context = new http4comm::Context (a_timeController, (Millisecond)2000);
}

void http4comm::Application::run ()
   throw (RuntimeException)
{
   a_communicator->accept ();
}

