// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_util_Tokenizer_hpp
#define anna_core_util_Tokenizer_hpp

#include <string>

#include <anna/core//DataBlock.hpp>

namespace anna {

class RuntimeException;

/**
   Tokenize the input string into several elements
*/
class Tokenizer {

  int _apply(const char* str, const char* separator) throw(RuntimeException);

public:
  typedef char* const* const_iterator;

  /**
     Constructor.
  */
  Tokenizer();

  /**
     Constructor.

     @param str Cadena sobre la que aplicar la separacion.
     @param separator Caracteres que van a actuar como separador de las subcadenas contenidas en el
     primer parametro.
  */
  Tokenizer(const char* str, const char* separator);

  /**
     Constructor.

     @param str Cadena sobre la que aplicar la separacion.
     @param separator Caracteres que van a actuar como separador de las subcadenas contenidas en el
     primer parametro.
  */
  Tokenizer(const std::string& str, const char* separator);

  /**
   * Destructor.
   */
  virtual ~Tokenizer();

  // Accesores
  /**
     Devuelve el estado del indicador de activacion de eliminacion de espacios de los extremos.
     \return El estado del indicador de activacion de eliminacion de espacios de los extremos.
  */
  bool activateStrip() const throw() { return a_activateStrip; }

  // Operadores

  /**
     Activa y/o desactiva que activa el sistema que permite recoger los elementos retornadodos
     por esta clase sin espacios por delante y por detras.
     \param _activateStrip Parametro que indica el estado de activacion o desactivacion.
  */
  void activateStrip(const bool _activateStrip) throw() { a_activateStrip = _activateStrip; }

  /**
    @return El elemento que ocupa la posicion i-esima.
    \warning Este mtodo tiene una eficiencia de O(1), mejor usar iteradores.
  */
  const char* at(const int i) throw(RuntimeException);

  /**
    @return El elemento que ocupa la posicion i-esima.
    \warning Este mtodo tiene una eficiencia de O(1), mejor usar iteradores.
  */
  const char* at(const int i) const throw(RuntimeException);

  /**
    @return El elemento que ocupa la posicion i-esima.
    \warning Este mtodo tiene una eficiencia de O(1), mejor usar iteradores.
  */
  const char* operator [](const int i) throw(RuntimeException) { return at(i); }

  /**
    @return El elemento que ocupa la posicion i-esima.
    \warning Este mtodo tiene una eficiencia de O(1), mejor usar iteradores.
  */
  const char* operator [](const int i) const throw(RuntimeException) { return at(i); }

  /**
     Process the separation over the string str with the separator provided.

     Internally used strtok_r has these imitations: sequence of two or more contiguous delimiter
     bytes in the parsed string is considered to be a single delimiter. Delimiter bytes at the start
     or end of the string are ignored. Put another way: the tokens returned by strtok() are always
     nonempty strings. To override these limitations, the string provided can be internally modified
     inserting a artificial token in order to cheat on strtok_r. For this feature, you may provide
     '<null>' or whatever string (non-empty) you prefer.

     @param str String to apply the separation.
     @param separator Characters used as separator within the string tokenized
     @param tokenizeContiguous If provided, it will be the artificial token used internally. The
            resulting tokens shall store this string in case of contiguous separators. NULL by
            default (original strtok_r behaviour).

     @return Number of tokens
  */
  int apply(const char* str, const char* separator, const char *tokenizeContiguous = NULL) throw(RuntimeException);
  int apply(const std::string& str, const char* separator, const char *tokenizeContiguous = NULL) throw(RuntimeException) {
    return apply(str.c_str(), separator, tokenizeContiguous);
  }


  // Metodos
  /**
     @return El ultimo elemento obtenido la aplicar la separacion.
  */
  const char* last() const throw(RuntimeException);

  /**
   * Devuelve el nmero de elementos obtenidos en la separacin.
   * \return el nmero de elementos obtenidos en la separacin.
   */
  int size() const throw() { return a_maxItem; }

  /**
   * Devuelve el iterador el comiento de los elementos obtenidos por #apply
   * \return el iterador el comiento de los elementos obtenidos por #apply
   */
  const_iterator begin() const throw() { return a_items; }

  /**
   * Devuelve el iterador al final de los elementos obtenidos por #apply
   * \return el iterador al final de los elementos obtenidos por #apply
   */
  const_iterator end() const throw() { return a_items + a_maxItem; }

  /**
     Devuelve la cadena referenciada por el iterator recibido como parametro.
     \return la cadena referenciada por el iterator recibido como parametro.
  */
  static const char* data(const_iterator ii) throw() { return *ii; }

private:
  static const int MaxItem;

  anna::DataBlock a_dataBlock;
  bool a_activateStrip;
  char** a_items;
  int a_maxItem;

  static char* strip(char* str) throw();

  void indexException(const int index, const char* fromFile, const int fromLine) const throw(RuntimeException);
};

}

#endif
