// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_time_internal_Timezone_hpp
#define anna_time_internal_Timezone_hpp


// Standard
#include <string>
#include <stdlib.h>
#include <iostream>

extern char *tzname[2];

//------------------------------------------------------------------------------
//---------------------------------------------------------------------- #define
//------------------------------------------------------------------------------


namespace anna {

namespace time {

class Timezone {

  bool a_initialized;
  std::string a_value;
  bool a_unsetTZ;

public:

  Timezone() { a_initialized = false; };
  ~Timezone() {};


  // sets
  // pass NULL to unset
  void set(const char *input) {
    a_initialized = true;
    a_value = input ? input : "";

  //  /*
  //    In many C libraries, TZ='' is equivalent to TZ='GMT0' or TZ='UTC0',
  //    and that's the behavior you're observing. An unset TZ is equivalent
  //    to a system-supplied default, typically the local time where the host
  //    is physically located.
  //  */
  //  if (input && (a_value == "")) a_value="UTC0";

    a_unsetTZ = !input;
  }

  Timezone & operator = (const Timezone &tz) {
    if(this == &tz) return (*this);

    a_initialized = tz.isInitialized();
    a_value = tz.getValue();
    a_unsetTZ = tz.unsetTZ();
    return (*this);
  }

  // gets

  // Operators

  friend bool operator == (const Timezone & tz1, const Timezone & tz2) {
    if(tz1.isInitialized() != tz2.isInitialized())
      return (false);

    if(tz1.unsetTZ())
      return (tz2.unsetTZ());

    return (tz1.getValue() == tz2.getValue());
  }

  friend bool operator != (const Timezone & tz1, const Timezone & tz2) {
    return (!(tz1 == tz2));
  }

  // be careful: empty if unset
  const std::string & getValue(void) const throw() {
    return (a_value);
  }


  bool isInitialized(void) const throw() {
    return (a_initialized);
  }

  //Unset is when getenv("TZ") returns NULL. Then, /etc/localtime used to be a copy or symlink to
  // any of the /usr/share/zoneinfo available timezones.
  bool unsetTZ(void) const throw() { return (a_unsetTZ); }

  bool apply() const throw() {
    //    http://stackoverflow.com/questions/5873029/questions-about-putenv-and-setenv
    if(unsetTZ()) {
      unsetenv("TZ");
    }
    else {
      setenv("TZ", a_value.c_str(), 1 /* overwrite */);

      /*
      // If TZ has an invalid setting, tzset() uses GMT0:
      // Check it:
      tzset();
      //std::cout << "tzname[0]: " << tzname[0] << std::endl;
      //std::cout << "tzname[1]: " << tzname[1] << std::endl;
      std::string tzname_1 = tzname[1];
      bool success = (tzname_1 != "");
      if (!success) std::cerr << "Invalid TZ '" << a_value << "'. Probably GMT0 is going to be used instead !" << std::endl;
      return success;
      */
    }

    return true;
  }

  std::string asString() const throw() {
    std::string result = getValue();
    if (unsetTZ()) return "[TZ unset]";
    if (result == "") return "[TZ empty]";
    return result;
  }
};


}
}

#endif

