// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_timex_Timer_hpp
#define anna_timex_Timer_hpp

#include <anna/timex/TimeEvent.hpp>

namespace anna {

namespace timex {

/**
   Clase base para los temporizadores. Se activa una unica vez y cuando transcurren los milisegundos
   establecidos por TimeEvent::getTimeout se invoca al metodo #expire. La implementacion de este metodo
   sera particular para cada uno de los temporizadores.

   @see Clock
*/
class Timer : public TimeEvent {
public:
  /**
     Constructor

     @param name Nombre logico de este temporizador.
     @param timeout Duracion del temporizador indicado en milisegundos.
  */
  Timer(const char* name, const Millisecond & timeout) :
    TimeEvent(anna_ptrnumber_cast(this), timeout),
    a_name(name) {
  }

  /**
     Devuelve una cadena con la informacion referente a este temporizador.
     \return Una cadena con la informacion referenta a este temporizador.
  */
  std::string asString() const
  throw() {
    std::string msg("Timer { ");
    msg += TimeEvent::asString();
    msg += " | Name: ";
    msg += a_name;
    return msg += " }";
  }

private:
  std::string a_name;
};

}
}

#endif


