// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/config/defines.hpp>

#include <anna/comm/SureTransport.hpp>
#include <anna/comm/Socket.hpp>
#include <anna/comm/Message.hpp>
#include <anna/comm/functions.hpp>

using namespace anna;

comm::TransportFactoryImpl <comm::SureTransport> comm::SureTransport::st_transportFactory;

comm::SureTransport::SureTransport() :
  comm::Transport(),
  a_precodec(true) {
  setInputMessage(new Message());
  char aux [sizeof(short int)];
  a_precodec.append(functions::codeShort(aux, initTag), sizeof(short int));
}

comm::SureTransport::~SureTransport() {
  delete getInputMessage();
}

int comm::SureTransport::calculeSize(const DataBlock& dataBlock)
throw(RuntimeException) {
  const char* buffer = dataBlock.getData();
  int result = -1;

  if(dataBlock.getSize() >= headerSize)
    if(buffer [0] == (char) 0xaa  && buffer [1] == (char) 0xaa)
      result = functions::decodeInteger(buffer + sizeof(short int));

  return result;
}

//-------------------------------------------------------------------------------------------
// Recordar que 'message' esta alojado en la memoria intermedia que el comm::ClientSocket
// tiene reservada para ir guardando los mensajes en proceso => no es una copia si no una
// referencia un puntero.
//
// (1) Transfiere la referencia de message al cuerpo del mensaje. Ojo!! tampoco hace copia.
//-------------------------------------------------------------------------------------------
const comm::Message* comm::SureTransport::decode(const DataBlock& message)
throw(RuntimeException) {
  const int totalSize = message.getSize();

  if(totalSize < headerSize)
    throw RuntimeException("Invalid message to decode", ANNA_FILE_LOCATION);

  const char* buffer = message.getData();

  if(buffer [0] != (char) 0xaa || buffer [1] != (char) 0xaa)
    throw RuntimeException("Missing start label on message (0xaaaa)", ANNA_FILE_LOCATION);

  const int bodySize = totalSize - headerSize;

  if(bodySize < 0)
    throw RuntimeException("Invalid message size to decode", ANNA_FILE_LOCATION);

  return getInputMessage()->setBody(buffer + headerSize, bodySize);             // (1)
}

//---------------------------------------------------------------------------
// Los componentes del SureTransport:
// Etiquta de comienzo 0xaaaa + short (<longitud ttal>) + datos
//---------------------------------------------------------------------------
const DataBlock& comm::SureTransport::code(comm::Message& message)
throw(RuntimeException) {
  char aux [sizeof(int)];
  const DataBlock& data = message.code();
  a_forCode = a_precodec;
  a_forCode.append(functions::codeInteger(aux, headerSize + data.getSize()), sizeof(int));
  return a_forCode += data;
}

