// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <string>

#include <anna/core/tracing/Logger.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>

#include <anna/timex/Engine.hpp>

#include <anna/core/oam/CounterScope.hpp>
#include <anna/core/oam/Counter.hpp>

using namespace std;
using namespace anna;

#define test_range(index) \
   if (index < 0 || index >= MaxCounter) { \
      string msg (asString ()); \
      msg += functions::asText (" | Index: ", index); \
      msg += functions::asText (" | Out of range [0, MaxCounter): ", index); \
      throw RuntimeException (msg, __FILE__, __LINE__); \
   }

#define test_instance(index) \
   if (a_counters [index] == NULL) { \
      string msg (asString ()); \
      msg += functions::asText (" | CounterId: ", index); \
      msg += " | Counter Id is not defined yet"; \
      throw RuntimeException (msg, __FILE__, __LINE__); \
   }


oam::CounterScope::~CounterScope() {
  for(int i = 0; i < MaxCounter; i ++) {
    if(a_counters [i] != NULL)
      delete a_counters [i];
  }
}

void oam::CounterScope::create(const int counter, const char* name)
throw(RuntimeException) {
  test_range(counter);

  if(a_counters [counter] != NULL) {
    string msg("Counter '");
    msg += name;
    msg += "' (";
    msg += functions::asString(counter);
    msg += ") already registered as ";
    msg += a_counters [counter]->asString();
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  a_counters [counter] = new Counter(*this, counter, name);
  LOGDEBUG(
    string msg("Creation | ");
    msg += asString();
    msg += " | ";
    msg += a_counters [counter]->asString();
    Logger::write(Logger::Debug, msg, ANNA_FILE_LOCATION)
  );
}

oam::Counter::type_t oam::CounterScope::increment(const int counter, const oam::Counter::type_t value)
throw(RuntimeException) {
  test_range(counter);
  test_instance(counter);
  a_counters [counter]->a_value += value;
  a_counters [counter]->a_accValue += value;
  LOGINFORMATION(a_counters [counter]->debug());
  return a_counters [counter]->a_value;
}

oam::Counter::type_t oam::CounterScope::assign(const int counter, const oam::Counter::type_t value)
throw(RuntimeException) {
  test_range(counter);
  test_instance(counter);
  a_counters [counter]->a_value = value;
  a_counters [counter]->a_accValue = value;
  LOGINFORMATION(a_counters [counter]->debug());
  return value;
}

oam::Counter::type_t oam::CounterScope::getValue(const int counter) const
throw(RuntimeException) {
  test_range(counter);
  test_instance(counter);
  return a_counters [counter]->a_value;
}

U64 oam::CounterScope::getAccValue(const int counter) const
throw(RuntimeException) {
  test_range(counter);
  test_instance(counter);
  return a_counters [counter]->a_accValue;
}

int oam::CounterScope::resetAccValues() throw(RuntimeException) {
  int result = 0; // affected counters

  for(int ii = 0; ii < MaxCounter; ii ++)
    if(a_counters [ii])
      result += (a_counters [ii]->resetAcc()) ? 1 : 0;

  return result;
}

const oam::Counter* oam::CounterScope::getCounter(const int counter) const
throw(RuntimeException) {
  test_range(counter);
  test_instance(counter);
  return a_counters [counter];
}

string oam::CounterScope::asString() const
throw() {
  string result("oam::CounterScope { Id: ");
  result += functions::asString(a_id);
  result += " | Name: ";
  result += a_name;
  return result += " }";
}

xml::Node* oam::CounterScope::asXML(xml::Node* parent) const
throw(RuntimeException) {
  xml::Node* result = parent->createChild("Scope");
  xml::Node* counter;

  for(int ii = 0; ii < MaxCounter; ii ++) {
    if(a_counters [ii] == NULL)
      continue;

    counter = result->createChild("Counter");
    counter->createAttribute("Id", ii);
    counter->createAttribute("Name", a_counters [ii]->getName());
    counter->createAttribute("Value", a_counters [ii]->getValue());
    counter->createAttribute("AccValue", a_counters [ii]->getAccumulatedValue());
  }

  return result;
}

