// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <stdlib.h>

#include <anna/core/tracing/Logger.hpp>

#include <anna/http/Request.hpp>

#include <anna/http/wims20/ServerSide.hpp>

using namespace std;
using namespace anna;

/*
 * (1) Si estamos esperando http://xxx/zzzz hay que verificar que no vamos a dar por buena una
 * expresin con la forma: http://xxx/zzzzAAAAAA, p.e.
 */
void http::wims20::ServerSide::decode(const http::Request& request)
throw(RuntimeException) {
  Abstract::clear();
  const string& uri = request.getURI();
  // Calcula la parte corta que hemos definido para el servicio
  const string& shortFixedPart = Abstract::calculeShortFixedPart();
  const int fixedLen = shortFixedPart.length();
  bool isOk = true;

  if(uri.compare(0, fixedLen, shortFixedPart) != 0)
    isOk = false;
  else if(uri [fixedLen] != '/')                                                  // (1)
    isOk = false;

  if(isOk == false) {
    string msg("http::wims20::ServerSide::decode | URI: ");
    msg += uri;
    msg += " | URI does not match with service '";
    msg += shortFixedPart;
    msg += "'" ;
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  // http://domain-openapis/path-openapis/serviceID/guid/other_possible_levels?query_parameters
  // Quita toda la parte "http://domain-openapis/path-openapis"
  string hierarchyAndParameter = uri.substr(fixedLen + 1);
  // Separa la jerargua de los parmetros (si los hay).
  const Tokenizer& tthp = split(SplitCode::HierarchyAndParameter, hierarchyAndParameter);
  const int size = tthp.size();

  if(size == 0 || size > 2) {
    string msg("URI '");
    msg += uri;
    msg += "' is not valid";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  try {
    switch(size) {
    case 1:
      decodeHierarchy(tthp [0]);
      break;
    case 2:
      decodeHierarchy(tthp [0]);
      decodeParameters(tthp [1]);
      break;
    }
  } catch(RuntimeException& ex) {
    string msg("URI '");
    msg += uri;
    msg += "' is not valid | ";
    msg += ex.getText();
    throw RuntimeException(msg, ex.getFromFile(), ex.getFromLine());
  }

  LOGDEBUG(
    string msg("http::wims20::ServerSide::decode | URI: ");
    msg += uri;
    msg += " | Result: ";
    msg += asString();
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
}

const string* http::wims20::ServerSide::getValue(const char* name, const Exception::Mode::_v mode) const
throw(RuntimeException) {
  const string* result = NULL;

  if(hasParameters() == true) {
    for(const_parameter_iterator ii = parameter_begin(), maxii = parameter_end(); ii != maxii; ii ++) {
      if(parameter_name(ii) == name) {
        const string& value = parameter_value(ii);
        result = &value;
        break;
      }
    }
  }

  if(result == NULL && (mode == Exception::Mode::Throw || mode == Exception::Mode::Trace)) {
    string msg(asString());
    msg += " | Parameter '";
    msg += name;
    msg += "' not found";
    RuntimeException ex(msg, ANNA_FILE_LOCATION);

    if(mode == Exception::Mode::Throw)
      throw ex;

    ex.trace();
  }

  return result;
}

const char* http::wims20::ServerSide::getCStringValue(const char* name, const Exception::Mode::_v mode) const
throw(RuntimeException) {
  const string* temp = getValue(name, mode);
  return (temp == NULL) ? NULL : temp->c_str();
}

int http::wims20::ServerSide::getIntegerValue(const char* name, const Exception::Mode::_v mode) const
throw(RuntimeException) {
  const string* tmp = getValue(name, mode);

  if(tmp == NULL)
    return 0;

  const char* value = tmp->c_str();
  return (anna_strncmp(value, "0x", 2) == 0) ? strtol(value + 2, NULL, 16) :  atoi(value);
}

/* Nos ha debido llegar algo asi como: serviceID/guid {other_possible_levels}
 */
void http::wims20::ServerSide::decodeHierarchy(const std::string& hierarchy)
throw(RuntimeException) {
  const Tokenizer& items = split(SplitCode::HierarchyItem, hierarchy);
  Abstract::setServiceID(items [0]);
  Abstract::setGUID(items [1]);

  if(items.size() > 2) {
    Tokenizer::const_iterator maxii, ii = items.begin();
    ii ++;
    ii ++;

    for(maxii = items.end(); ii != maxii; ii ++)
      Abstract::other_level_add(Tokenizer::data(ii));
  }
}

/* Nos ha debido llegar algo as como: name=value&*{nameN=valueN}
 */
void http::wims20::ServerSide::decodeParameters(const std::string& parameters)
throw(RuntimeException) {
  const Tokenizer& tkparams = split(SplitCode::Parameters, parameters);

  for(Tokenizer::const_iterator ii = tkparams.begin(), maxii = tkparams.end(); ii != maxii; ii ++) {
    const string& parameter = Tokenizer::data(ii);
    // Separa los XXX=YYYY
    const Tokenizer& tkparam = split(SplitCode::ParameterAndArgument, parameter);
    Abstract::parameter_set(tkparam [0], tkparam [1]);
  }
}

const Tokenizer& http::wims20::ServerSide::split(const SplitCode::_v splitZone, const std::string& str)
throw(RuntimeException) {
  static const char* separator [] = { "?", "/", "&", "=", NULL };
  a_tokenizer [splitZone].apply(str, separator [splitZone]);
  LOGDEBUG(
    string msg("String: ");
    msg += str;
    msg += " | Separator: ";
    msg += separator [splitZone];
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  return a_tokenizer [splitZone];
}

