// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <sys/types.h>
#include <sys/stat.h>

#include <poll.h>

#include <anna/core/util/Tokenizer.hpp>
#include <anna/config/defines.hpp>
#include <anna/core/functions.hpp>

#include <anna/io/functions.hpp>

using namespace std;
using namespace anna;

void io::functions::mkdir(const std::string& path)
throw(RuntimeException) {
  Tokenizer tokenizer;
  string w;
  int r;
  tokenizer.apply(path, "/");

  for(Tokenizer::const_iterator ii = tokenizer.begin(), maxii = tokenizer.end(); ii != maxii; ii ++) {
    w += "/";
    w += Tokenizer::data(ii);

    if(io::functions::exists(w) == false) {
      anna_signal_shield(r, ::mkdir(w.c_str(), S_IRWXU | S_IRWXG));

      if(r == -1)
        throw RuntimeException(path, errno, ANNA_FILE_LOCATION);
    }
  }
}

bool io::functions::exists(const char* path)
throw(RuntimeException) {
  struct stat data;
  int r;
  anna_signal_shield(r, stat(path, &data));

  if(r != -1)
    return true;

  if(errno == ENOENT)
    return false;

  throw RuntimeException(path, errno, ANNA_FILE_LOCATION);
}

bool io::functions::isADirectory(const char* path)
throw(RuntimeException) {
  struct stat data;
  int r;
  anna_signal_shield(r, stat(path, &data));

  if(r != 0)
    throw RuntimeException(path, errno, ANNA_FILE_LOCATION);

  return ((data.st_mode & S_IFDIR) != 0);
}

ino_t io::functions::getINode(const char* path)
throw(RuntimeException) {
  struct stat data;
  int r;
  anna_signal_shield(r, stat(path, &data));

  if(r != 0)
    throw RuntimeException(path, errno, ANNA_FILE_LOCATION);

  return data.st_ino;
}

/*static*/
bool io::functions::waitInput(const int fd, const Millisecond &timeout)
throw(RuntimeException) {
  pollfd waiting;
  int r;
  waiting.fd = fd;
  waiting.events = POLLIN;
  anna_signal_shield(r, poll(&waiting, 1, timeout));

  if(r == -1) {
    string msg("io::functions::waitInput: ");
    msg += anna::functions::asString(fd);
    throw RuntimeException(msg, errno, ANNA_FILE_LOCATION);
  }

  return (r != 0);
}

