// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <iostream>

#include <anna/core/mt/Guard.hpp>
#include <anna/core/functions.hpp>
#include <anna/core/tracing/Logger.hpp>

#include <anna/comm/ClientSocket.hpp>
#include <anna/comm/CongestionController.hpp>

#include <anna/test/Communicator.hpp>

using namespace std;
using namespace anna;

bool test::Communicator::canContinue (const comm::ClientSocket& clientSocket)
   throw (RuntimeException)
{
   Guard guard (this, "test::Communicator::canContinue");

   if (a_initTime == 0)
      a_initTime = anna::functions::millisecond ();

   if (a_messageCounter > 0)
      if (a_messageCounter == a_maxMessage && hasRequestedStop () == false) {
         terminate ();
         return false;
      }

   using namespace anna::comm;

   CongestionController& congestionController = CongestionController::instantiate ();

   a_messageCounter ++;

   if (congestionController.getAdvice (clientSocket) == CongestionController::Advice::Discard)
      return false;

   a_successCounter ++;

   return true;
}

void test::Communicator::delay ()
   throw (RuntimeException)
{
  if (a_delay > 0) {
      int random = (a_delay > 10) ? (rand () % (a_delay / 10)): 0;
      int sign = rand () % 2;
   
      if (sign == 0)
         random *= -1;

      const Microsecond init = anna::functions::hardwareClock ();

      anna::functions::sleep (a_delay + (Millisecond)random);

      if (true) {
         Guard guard (this, "test::Communicator::delay");   
         a_avgDelay += anna::functions::hardwareClock () - init;
      }
   }
}

void  test::Communicator::terminate ()
   throw ()
{
   if (hasRequestedStop () == true)
      return;

   requestStop ();

   const Millisecond serviceTime = anna::functions::millisecond () - a_initTime;

   LOGNOTICE (
      const int workload = (serviceTime == 0) ? 0: a_messageCounter * 1000 / serviceTime;   
      string msg (anna::functions::asText ("Tiempo de servicio: ", (int) serviceTime));
      msg += anna::functions::asText (" ms | Carga: ", workload);
      msg += anna::functions::asText (" msg/seg | Mensajes recibidos: ", a_messageCounter);
      msg += anna::functions::asText (" | Mensajes tratados: ", a_successCounter);
      Logger::notice (msg, ANNA_FILE_LOCATION);      
      cout << msg << endl << endl;
      
      msg = "Retardo medio (us) | ";
      msg += a_avgDelay.asString ();
      Logger::notice (msg, ANNA_FILE_LOCATION);      
      cout << msg << endl << endl;
   );
}

//----------------------------------------------------------------------------------------
// (1) Dos handlers, uno el ServerSocket y otro el ClientSocket que va a cerran =>
// si es asi termina
//----------------------------------------------------------------------------------------
void test::Communicator::eventOverQuota (const comm::ClientSocket&) 
   throw ()
{
   int counter = 0;

   for (handler_iterator ii = handler_begin (), maxii = handler_end (); ii != maxii; ii ++)
      counter ++;

   LOGNOTICE (
      Logger::notice (anna::functions::asText ("Handlers: ", counter), ANNA_FILE_LOCATION);
   );
      
   if (counter == 2)  // (1)
      terminate ();
}
